package tech.nevets.sfss;

import javax.net.ssl.*;
import java.io.*;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.security.spec.PKCS8EncodedKeySpec;

public class StoreManager {
    public static final File PRIVATE_KEY = new File("./certificates/private.key");
    public static final File PRIVATE_PKCS8 = new File("./certificates/private_key_pkcs8.der");
    public static final File PUBLIC_CERTIFICATE = new File("./certificates/public.cer");
    private static SSLContext sslCtx = null;
    public static volatile boolean certificatesLoaded = false;


    public static void generateCertificates() {

        if (PRIVATE_KEY.exists() && PUBLIC_CERTIFICATE.exists() && PRIVATE_PKCS8.exists()) {
            return;
        }

        String[][] cmds = {
                {"openssl", "genrsa", "-out", PRIVATE_KEY.getAbsolutePath(), "2048"},
                {"openssl", "req", "-new", "-x509", "-config", "./openssl.cnf", "-key", PRIVATE_KEY.getAbsolutePath(), "-out", PUBLIC_CERTIFICATE.getAbsolutePath(), "-days", "365"},
                {"openssl", "pkcs8", "-topk8", "-inform", "PEM", "-outform", "DER", "-in", PRIVATE_KEY.getAbsolutePath(), "-out", PRIVATE_PKCS8.getAbsolutePath(), "-nocrypt"}
        };
        new File("./certificates").mkdirs();
        for (String[] cmd : cmds) {
            Process ps;
            try {
                ps = Runtime.getRuntime().exec(cmd);
                int exitCode = ps.waitFor();
                if (exitCode != 0) {
                    System.err.println("Error: Process exited with code " + exitCode);
                    return;
                }
            } catch (Exception e) {
                e.printStackTrace();
                return;
            }
        }
    }

    public static void loadCertificates() {
        try {
            KeyStore keyStore = KeyStore.getInstance(KeyStore.getDefaultType());
            keyStore.load(null);

            // Load the certificate
            CertificateFactory certFactory = CertificateFactory.getInstance("X.509");
            try (InputStream certStream = new FileInputStream(PUBLIC_CERTIFICATE)) {
                X509Certificate cert = (X509Certificate) certFactory.generateCertificate(certStream);
                keyStore.setCertificateEntry("certificate", cert);
            }

            // Load the private key
            try (FileInputStream fis = new FileInputStream(PRIVATE_PKCS8)) {
                KeyFactory keyFactory = KeyFactory.getInstance("RSA");
                byte[] keyBytes = fis.readAllBytes();
                PKCS8EncodedKeySpec spec = new PKCS8EncodedKeySpec(keyBytes);
                PrivateKey privateKey = keyFactory.generatePrivate(spec);
                keyStore.setKeyEntry("privateKey", privateKey, new char[0], new Certificate[]{keyStore.getCertificate("certificate")});
            }

            KeyManagerFactory kmf = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
            kmf.init(keyStore, new char[0]);

            TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
            tmf.init(keyStore);

            sslCtx = SSLContext.getInstance("TLS");
            sslCtx.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
            certificatesLoaded = true;
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    public static SSLContext getSSLContext() {
        return certificatesLoaded ? sslCtx : null;
    }
}
